@@ -31,7 +31,8 @@
 
 #ifdef HAVE_RFC40
 
-KEARasterAttributeTable::KEARasterAttributeTable(kealib::KEAAttributeTable *poKEATable)
+KEARasterAttributeTable::KEARasterAttributeTable(kealib::KEAAttributeTable *poKEATable, 
+                        KEARasterBand *poBand)
 {
     for( size_t nColumnIndex = 0; nColumnIndex < poKEATable->getMaxGlobalColIdx(); nColumnIndex++ )
     {
@@ -48,6 +49,7 @@ KEARasterAttributeTable::KEARasterAttributeTable(kealib::KEAAttributeTable *poKE
         m_aoFields.push_back(sKEAField);
     }
     m_poKEATable = poKEATable;
+    m_poBand = poBand;
 }
 
 KEARasterAttributeTable::~KEARasterAttributeTable()
@@ -928,6 +930,39 @@ CPLErr KEARasterAttributeTable::CreateColumn( const char *pszFieldName,
     return CE_None;
 }
 
+CPLErr KEARasterAttributeTable::SetLinearBinning( double dfRow0Min,
+                                            double dfBinSize )
+{
+    size_t nRows = m_poKEATable->getSize();
+
+    osWorkingResult.Printf( "%.16g", dfRow0Min);
+    m_poBand->SetMetadataItem("STATISTICS_HISTOMIN", osWorkingResult);
+    osWorkingResult.Printf( "%.16g", (nRows - 1) * dfBinSize + dfRow0Min);
+    m_poBand->SetMetadataItem("STATISTICS_HISTOMAX", osWorkingResult);
+
+    // Below no longer necessary as there is a STATISTICS_HISTONUMBINS
+    // metadata item that is set from the size of the RAT
+    //osWorkingResult.Printf( "%lu", (unsigned long)nRows);
+    //m_poBand->SetMetadataItem("STATISTICS_HISTONUMBINS", osWorkingResult);
+
+    return CE_None;
+}
+
+int KEARasterAttributeTable::GetLinearBinning( double *pdfRow0Min,
+                                            double *pdfBinSize ) const
+{
+    const char *pszMin = m_poBand->GetMetadataItem("STATISTICS_HISTOMIN");
+    const char *pszMax = m_poBand->GetMetadataItem("STATISTICS_HISTOMAX");
+    if( ( pszMin == NULL ) || ( pszMax == NULL ) )
+    {
+        return FALSE;
+    } 
+    *pdfRow0Min = atof(pszMin);
+    *pdfBinSize = (atof(pszMax) - *pdfRow0Min) / (m_poKEATable->getSize() - 1);
+
+    return TRUE;
+}
+
 CPLXMLNode *KEARasterAttributeTable::Serialize() const
 {
     if( ( GetRowCount() * GetColumnCount() ) > RAT_MAX_ELEM_FOR_CLONE )
